/*
	ProWesS, PROGS Windowing System

	PWActivate

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 14 January 1995
	08.11.1995 removed an unnecessary PWRedraw call
	25.11.1995 added support for SSTATUS_PAGESHOW
	06.01.1996 ActionInit support added
	04.02.1996 added support for EVENT_TOUCH_EDGE
			   added support for software generated events
	06.02.1996 fixed some problems with software events
	23.03.1996 fixed problem that border round current item remains visible
			   when item sizes are redetermined
	13.03.1996 software events and wake now properly trapped (thanks to Tony
			   Tebby). This also makes code bit easier and shorter.
	25.05.1996 proper return of error on PWHandleRegion
	08.01.1996 PWActivate(NULL) now possible
	12.02.1997 PW_EVENT_CATCHEND caused when exitting window
	14.04.1997 changed event handling to "better" order
*/

#include <job.h>
#include <win.h>

#include "private.h"
#include "PWHandler.h"

extern short system_scaleborderwidth;
extern short XWindowSize, YWindowSize;

#define catch(x) do { if (err=(x)) return err; } while (0)

#define EXIT goto exit;

Error PWActivate(PWObject object)
{
	Error err;
	PWSystem *system;
	char key, firstcall=TRUE, following;

	if (object==NULL) return ERR_IPAR;
	system=object->event.region->system;

	if (system->regions==NULL) return ERR_RWF;
	if (system->activated) return ERR_ISYN;

	system->status|=SSTATUS_WINCHANGE;	/* make sure window is drawn */
	system->status&=~SSTATUS_BREAKDOWN;
	system->active=NULL;				/* no current object */

	while (system->status&(SSTATUS_WINCHANGE|SSTATUS_SLEEP))
	{
		if (system->status&SSTATUS_REMOVE)
			err=PWRemoveIndicated(system);

		if (system->status&SSTATUS_SLEEP)
		{
			err=PWSleep(system);
			if (err) EXIT;
		}
		system->activated=TRUE;

		err=PWDetermineSize(system);
		if (err) EXIT;
		err=PWRedraw(system);
		if (err) EXIT;

		if (system->catchkeys && system->catchkeys->type->PWHandle)
			system->catchkeys->type->PWHandle(system->catchkeys,PW_EVENT_CATCHSTART);

		if (firstcall)
		{
			PWObject object; PWRegion *region;

			WINPointerPositionGet(system->window,
								  &system->ptr_xpos,&system->ptr_ypos);
			firstcall=FALSE;

			if (system->ActionInit)
			{
				system->status&=~SSTATUS_PAGESHOW;
				for (region=system->regions ;
					 !(object=region->owner) && !(object=region->parentObject) ;
					 region=region->children);
				err=system->ActionInit(object);
				if (err) EXIT;
			}
		}

		system->ptr_info.event=0;
		system->ptr_info.keystroke=0;
		following=FALSE;

		while (!(system->status&(SSTATUS_BREAKDOWN|SSTATUS_WINCHANGE)))
		{
			err=PWRedraw(system);
			if (err) EXIT;

			if (following)
			{
				err=WINPointerRead(system->window, FOREVER,
									EVENT_MOVE|EVENT_KEYSTROKE|
									EVENT_TOUCH_EDGE|EVENT_LEVELS,
								   &system->ptr_xpos,&system->ptr_ypos,&system->ptr_info);
				if (err) EXIT;
			}
			following=TRUE;

			err=PWHandleRegion(system,NULL);
			if (err) EXIT;

			if (key=system->ptr_info.keystroke)
			{
				if (key==KEYSTROKE_HIT)
				{
					/* possible window move */

					/* test if the hit was inside the scaleborder */
					if (system->ptr_info.subwindow>=0)
					{
						err=PWWindowMove(system,system->ptr_info.subwindow&-2);
						if (err) EXIT;
					}
					else
					{
						err=PWHandleKeyPress(system);
						if (err) EXIT;
					}
				}
				else if (key==KEYSTROKE_DO)
				{
					/* possible window size */

					/* test if the hit was inside the scaleborder */
					if (system->ptr_info.subwindow>=0)
					{
						/* only scale if scaling is possible, else just move */
						if (system->status&SSTATUS_NOXSCALE && system->status&SSTATUS_NOYSCALE)
							err=PWWindowMove(system,system->ptr_info.subwindow&-2);
						else
							err=PWWindowScale(system,system->ptr_info.subwindow&-2);
						if (err) EXIT;
					}
					else
					{
						err=PWHandleKeyPress(system);
						if (err) EXIT;
					}
				}
				else
				{
					err=PWHandleKeyPress(system);
					if (err) EXIT;
				}
				system->ptr_info.keystroke=0;
			}
			if (system->ptr_info.event & EVENT_LEVELS)
			{
				err=SoftwareEvent(system);
				if (err) EXIT;
			}

			if (system->ptr_info.event & EVENT_TOUCH_EDGE)
				PWScrollTouch(system);
		}
		if (system->catchkeys && system->catchkeys->type->PWHandle)
			system->catchkeys->type->PWHandle(system->catchkeys,PW_EVENT_CATCHEND);
		system->activated=FALSE;
	}

	system->status&=~SSTATUS_BREAKDOWN;
exit:
	if (system->activated)
	{
		if (system->catchkeys && system->catchkeys->type->PWHandle)
			system->catchkeys->type->PWHandle(system->catchkeys,PW_EVENT_CATCHEND);
		system->activated=FALSE;
	}
	/* remove the outline/window from screen */
	PWClose(system);
	return err;
}

Error SoftwareEvent(PWSystem *system)
{
	Error err=ERR_OK;
	PWObject object;
	PWRegion *region;

	for (region=system->regions ;
		 !(object=region->owner) && !(object=region->parentObject) ;
		 region=region->children);

	if (system->ptr_info.event & EVENT_LEVEL1 && system->EventLevel1)
		catch( system->EventLevel1(object) );
	if (system->ptr_info.event & EVENT_LEVEL2 && system->EventLevel2)
		catch( system->EventLevel2(object) );
	if (system->ptr_info.event & EVENT_LEVEL3 && system->EventLevel3)
		catch( system->EventLevel3(object) );
	if (system->ptr_info.event & EVENT_LEVEL4 && system->EventLevel4)
		catch( system->EventLevel4(object) );
	if (system->ptr_info.event & EVENT_LEVEL5 && system->EventLevel5)
		catch( system->EventLevel5(object) );
	if (system->ptr_info.event & EVENT_LEVEL6 && system->EventLevel6)
		catch( system->EventLevel6(object) );
	if (system->ptr_info.event & EVENT_LEVEL7 && system->EventLevel7)
		catch( system->EventLevel7(object) );
	if (system->ptr_info.event & EVENT_LEVEL8 && system->EventLevel8)
		catch( system->EventLevel8(object) );
	return err;
}
